use gtk::{
    CssProvider, EventControllerKey, Orientation,
    gdk::{Display, Key},
    glib,
    prelude::*,
};
use gtk4_layer_shell::{Edge, Layer, LayerShell};

const ITEMS: [&'static str; 3] = ["Test", "Example", "Placeholder"];

fn main() -> glib::ExitCode {
    let app = gtk::Application::builder()
        .application_id("nl.kallestruik.application-launcher")
        .build();
    app.connect_startup(|_| load_css());
    app.connect_activate(build_ui);
    app.run()
}

fn build_ui(application: &gtk::Application) {
    let window = gtk::ApplicationWindow::new(application);
    window.set_title(Some("Application Launcher"));
    window.init_layer_shell();
    window.set_keyboard_mode(gtk4_layer_shell::KeyboardMode::Exclusive);
    window.set_layer(Layer::Overlay);

    // This centers the window
    window.set_anchor(Edge::Top, false);
    window.set_anchor(Edge::Right, false);
    window.set_anchor(Edge::Bottom, false);
    window.set_anchor(Edge::Left, false);

    let container = gtk::Box::builder()
        .orientation(Orientation::Vertical)
        .spacing(6)
        .build();
    window.set_child(Some(&container));

    let input = gtk::Entry::builder()
        .css_classes(vec!["main-input"])
        .placeholder_text("Start typing...")
        .width_request(800)
        .build();

    container.append(&input);

    let result_container = gtk::Box::builder()
        .orientation(Orientation::Vertical)
        .spacing(2)
        .height_request(600)
        .build();
    container.append(&result_container);

    input.connect_changed(glib::clone! {
        #[weak]
        result_container,
        move |input| {
            remove_all_children(&result_container);
            let search: &str = &input.text().to_lowercase();
            for item in ITEMS {
                if item.to_lowercase().starts_with(&search) {
                    let label = gtk::Label::new(Some(item));
                    result_container.append(&label);
                }
            }
            println!("{}", input.text());
    }});

    let event_controller = EventControllerKey::new();
    event_controller.connect_key_pressed(|_, key, _, _| {
        match key {
            Key::Escape => {
                std::process::exit(0);
            }
            _ => (),
        }
        glib::Propagation::Proceed
    });

    window.add_controller(event_controller);

    window.show();
}

fn remove_all_children(b: &gtk::Box) {
    let mut child = b.first_child();
    while let Some(c) = child {
        b.remove(&c);
        child = b.first_child();
    }
}

fn load_css() {
    // Load the CSS file and add it to the provider
    let provider = CssProvider::new();
    provider.load_from_data(include_str!("style.css"));

    // Add the provider to the default screen
    gtk::style_context_add_provider_for_display(
        &Display::default().expect("Could not connect to a display."),
        &provider,
        gtk::STYLE_PROVIDER_PRIORITY_APPLICATION,
    );
}
